@@ -12,7 +12,8 @@ from transformers import (PreTrainedTokenizerBase,
                           PrinterCallback,
                           EarlyStoppingCallback,
                           RobertaForSequenceClassification,
-                          RobertaForTokenClassification)
+                          RobertaForTokenClassification,
+                          ProgressCallback)
 from typing import Dict, Optional, Tuple, List, Any
 import numpy as np
 import requests
@@ -28,7 +29,8 @@ import re
 import random
 
 from ...utils import (MODEL_CLASSES, is_module_installed, InvalidBenchmark,
-                      TwolabelTrainer, get_all_datasets)
+                      TwolabelTrainer, get_all_datasets,
+                      NeverLeaveProgressCallback)
 
 
 logger = logging.getLogger(__name__)
@@ -207,8 +209,8 @@ class BaseBenchmark(ABC):
                 model.
             framework (str or None, optional):
                 The framework the model has been built in. Currently supports
-                'pytorch', 'tensorflow', 'jax' and 'spacy'. If None then this
-                will be inferred from `model_id`. Defaults to None.
+                'pytorch' and 'spacy'. If None then this will be inferred from
+                `model_id`. Defaults to None.
             task (str or None, optional):
                 The task for which the model was trained on. If None then this
                 will be inferred from `model_id`. Defaults to None.
@@ -236,10 +238,6 @@ class BaseBenchmark(ABC):
                 import torch
                 import torch.nn as nn
                 from torch.nn import Parameter
-            elif framework == 'tensorflow':
-                import tensorflow  # noqa
-            elif framework == 'jax':
-                import flax  # noqa
             elif framework == 'spacy':
                 import spacy
 
@@ -256,7 +254,7 @@ class BaseBenchmark(ABC):
                    f'scandeval[{framework}]`.')
             raise ModuleNotFoundError(msg)
 
-        if framework in ['pytorch', 'tensorflow', 'jax']:
+        if framework == 'pytorch':
 
             if task == 'fill-mask':
                 params = dict(num_labels=self.num_labels,
@@ -491,7 +489,9 @@ class BaseBenchmark(ABC):
             if not is_module_installed(local_model_id):
                 url = (f'https://huggingface.co/{model_id}/resolve/main/'
                        f'{local_model_id}-any-py3-none-any.whl')
-                subprocess.run(['pip3', 'install', url])
+                logger.info('Model not installed. Downloading model')
+                subprocess.run(['pip3', 'install', url, '--quiet'])
+                logger.info('Finished downloading model. Resuming benchmark:')
 
             # Load the model
             try:
@@ -575,7 +575,7 @@ class BaseBenchmark(ABC):
     def _log_metrics(self,
                      metrics: Dict[str, List[Dict[str, float]]],
                      finetuned: bool,
-                     model_id: str):
+                     model_id: str) -> Dict[str, dict]:
         '''Log the metrics.
 
         Args:
@@ -583,10 +583,18 @@ class BaseBenchmark(ABC):
                 The metrics that are to be logged. This is a dict with keys
                 'train' and 'test', with values being lists of dictionaries
                 full of metrics.
-            ta
+            finetuned (bool):
+                Whether the model is finetuned or not.
             model_id (str):
                 The full HuggingFace Hub path to the pretrained transformer
                 model.
+
+        Returns:
+            dict:
+                A dictionary with keys 'raw_metrics' and 'total', with
+                'raw_metrics' being identical to `metrics` and 'total' being
+                a dictionary with the aggregated metrics (means and standard
+                errors).
         '''
         # Initial logging message
         if finetuned:
@@ -596,6 +604,9 @@ class BaseBenchmark(ABC):
             msg = (f'Finished evaluation of {model_id} on {self.name}.')
         logger.info(msg)
 
+        # Initialise the total dict
+        total_dict = dict()
+
         # Logging of the metric(s)
         for metric_key, metric_name in self.metric_names.items():
             scores = self._get_stats(metrics, metric_key)
@@ -612,8 +623,23 @@ class BaseBenchmark(ABC):
                 train_se *= 100
                 msg += f'\n  - Train: {train_score:.2f} ± {train_se:.2f}'
 
+                # Store the aggregated train metrics
+                total_dict[f'train_{metric_key}'] = train_score
+                total_dict[f'train_{metric_key}_se'] = train_se
+
+            # Store the aggregated test metrics
+            total_dict[f'test_{metric_key}'] = test_score
+            total_dict[f'test_{metric_key}_se'] = test_se
+
+            # Log the scores
             logger.info(msg)
 
+        # Define a dict with both the raw metrics and the aggregated metrics
+        extended_metrics = dict(raw_metrics=metrics, total=total_dict)
+
+        # Return the extended metrics
+        return extended_metrics
+
     @abstractmethod
     def _get_spacy_predictions_and_labels(self,
                                           model,
@@ -632,8 +658,7 @@ class BaseBenchmark(ABC):
         '''
         pass
 
-    @staticmethod
-    def _fetch_model_metadata(model_id: str) -> Dict[str, str]:
+    def _fetch_model_metadata(self, model_id: str) -> Dict[str, str]:
         '''Fetches metdataof a model from the HuggingFace Hub.
 
         Args:
@@ -668,9 +693,11 @@ class BaseBenchmark(ABC):
                       for a in a_tags_with_class
                       if 'tag-red' in a['class']]
 
+        # Set up the order of the frameworks
+        valid_frameworks = ['pytorch', 'spacy']
+
         # Extract a single valid framework in which the model has been
         # implemented
-        valid_frameworks = ['pytorch', 'tensorflow', 'jax', 'spacy']
         for valid_framework in valid_frameworks:
             if valid_framework in frameworks:
                 framework = valid_framework
@@ -694,7 +721,7 @@ class BaseBenchmark(ABC):
     def benchmark(self,
                   model_id: str,
                   progress_bar: bool = True
-                  ) -> Dict[str, List[Dict[str, float]]]:
+                  ) -> Dict[str, dict]:
         '''Benchmark a model.
 
         Args:
@@ -706,8 +733,9 @@ class BaseBenchmark(ABC):
 
         Returns:
             dict:
-                The keys in the dict are 'train' and 'test', and the values
-                contain the metrics for that given dataset split.
+                The keys in the dict are 'raw_metrics' and 'total', with all
+                the raw metrics in the first dictionary and the aggregated
+                metrics in the second.
 
         Raises:
             RuntimeError: If the extracted framework is not recognized.
@@ -733,7 +761,7 @@ class BaseBenchmark(ABC):
         # Get bootstrap sample indices
         test_bidxs = rng.integers(0, len(test), size=(9, len(test)))
 
-        if framework in ['pytorch', 'tensorflow', 'jax']:
+        if framework in ['pytorch']:
 
             # Set platform-dependent random seeds
             if framework == 'pytorch':
@@ -742,10 +770,6 @@ class BaseBenchmark(ABC):
                 torch.cuda.manual_seed_all(4242)
                 torch.backends.cudnn.benchmark = False
 
-            elif framework == 'tensorflow':
-                import tensorflow as tf
-                tf.random.set_seed(4242)
-
             # Extract the model and tokenizer
             model = model_dict['model']
             tokenizer = model_dict['tokenizer']
@@ -770,7 +794,7 @@ class BaseBenchmark(ABC):
             # Set up progress bar
             if finetune:
                 if progress_bar:
-                    itr = tqdm(range(10))
+                    itr = tqdm(range(10), desc='Benchmarking')
                 else:
                     itr = range(10)
             else:
@@ -847,6 +871,14 @@ class BaseBenchmark(ABC):
                         if not self.verbose:
                             trainer.remove_callback(PrinterCallback)
 
+                        # Remove the progress bar callback
+                        trainer.remove_callback(ProgressCallback)
+
+                        # Add the custom progress callback if `progress_bar` is
+                        # True
+                        if progress_bar:
+                            trainer.add_callback(NeverLeaveProgressCallback)
+
                         # Finetune the model
                         if finetune:
                             trainer.train()
@@ -859,8 +891,15 @@ class BaseBenchmark(ABC):
                             )
                             metrics['train'].append(train_metrics)
 
+                        # Set up a progress bar for the test datasets if we are
+                        # not finetuning
+                        if not finetune and progress_bar:
+                            test_itr = tqdm(tests, desc='Benchmarking')
+                        else:
+                            test_itr = tests
+
                         # Log test metrics
-                        for dataset in tests:
+                        for dataset in test_itr:
                             test_metrics = trainer.evaluate(
                                 dataset,
                                 metric_key_prefix='test'
@@ -908,7 +947,9 @@ class BaseBenchmark(ABC):
                             pass
                         gc.collect()
 
-            self._log_metrics(metrics, model_id=model_id, finetuned=finetune)
+            metrics = self._log_metrics(metrics=metrics,
+                                        model_id=model_id,
+                                        finetuned=finetune)
 
             # Garbage collection, to avoid memory issues
             try:
@@ -934,7 +975,7 @@ class BaseBenchmark(ABC):
 
             # Get the test predictions
             all_test_metrics = list()
-            for dataset in tests:
+            for dataset in tqdm(tests, desc='Benchmarking'):
                 preds_labels = self._get_spacy_predictions_and_labels(
                     model=model,
                     dataset=dataset,
@@ -978,7 +1019,9 @@ class BaseBenchmark(ABC):
                 all_train_metrics.append(train_metrics)
                 metrics['train'] = all_train_metrics
 
-            self._log_metrics(metrics, model_id=model_id, finetuned=False)
+            metrics = self._log_metrics(metrics=metrics,
+                                        model_id=model_id,
+                                        finetuned=False)
 
             # Garbage collection, to avoid memory issues
             try:
