@@ -187,7 +187,7 @@ class TokenClassificationBenchmark(BaseBenchmark, ABC):
         Returns:
             HuggingFace dataset: The preprocessed dataset.
         '''
-        if framework in ['pytorch', 'tensorflow', 'jax']:
+        if framework == 'pytorch':
             map_fn = partial(self._tokenize_and_align_labels,
                              tokenizer=kwargs['tokenizer'],
                              label2id=kwargs['config'].label2id)
@@ -233,7 +233,7 @@ class TokenClassificationBenchmark(BaseBenchmark, ABC):
         '''
         # Initialise progress bar
         if progress_bar:
-            itr = tqdm(dataset['doc'])
+            itr = tqdm(dataset['doc'], desc='Evaluating model', leave=False)
         else:
             itr = dataset['doc']
 
